(define-module (ui squidmill ajax)
  :use-module (srfi srfi-1)
  :use-module (srfi srfi-19)
  :use-module (ice-9 format)
  :use-module (alterator ajax)
  :use-module (alterator algo)
  :use-module (alterator plist)
  :use-module (alterator woo)
  :export (init))

(define *date-type* "today")
(define *sdate* #f)
(define *edate* #f)
(define *user-pat-type* "all")
(define *user* #f)
(define *user-pat* #f)
(define *uri-pat-type* "all")
(define *uri-pat* #f)
(define *size-type* "any")

(define (date-range sdate edate)
  (list 'sdate (or (and sdate (if (string? sdate)
                                sdate
                                (date->string sdate "~1")))
                   "")
        'edate (or (and edate (if (string? edate)
                                edate
                                (date->string edate "~1")))
                   "")))

(define (date-values)
  (case (string->symbol (form-value "date_type"))
    ((today) (let ((date (current-date)))
               (date-range date date)))
    ((week) (let* ((edate (current-date))
                   (sdate (let loop ((sdate edate))
                            (if (= (date-week-day sdate) 1)
                              sdate
                              (loop (julian-day->date
                                      (- (date->julian-day sdate)
                                         1)))))))
               (date-range sdate edate)))
    ((month) (let* ((edate (current-date))
                    (sdate (make-date 0 0 0 0 1
                                      (date-month edate)
                                      (date-year edate)
                                      (date-zone-offset edate))))
               (date-range sdate edate)))
    ((year) (let* ((edate (current-date))
                    (sdate (make-date 0 0 0 0 1 1
                                      (date-year edate)
                                      (date-zone-offset edate))))
               (date-range sdate edate)))
    ((period) (date-range (form-value "start_date")
                          (form-value "end_date")))
    (else (date-range #f #f))))

(define (user-values)
  (list 'user_pat
        (case (string->symbol (form-value "user_pat_type"))
          ((one) (form-value "user"))
          ((pattern) (form-value "user_pat"))
          (else ""))))

(define (uri-values)
  (list 'uri_pat
        (case (string->symbol (form-value "uri_pat_type"))
          ((pattern) (form-value "uri_pat"))
          (else ""))))

(define (size-range min-traf max-traf)
  (list 'min_traf (or (and min-traf (number->string min-traf)) "")
        'max_traf (or (and max-traf (number->string max-traf)) "")))

(define (size-values)
  (case (string->symbol (form-value "size_type"))
    ((100Gb+) (size-range (* 100 1024 1024 1024) #f))
    ((100Gb-) (size-range #f (* 100 1024 1024 1024)))
    ((10Gb-) (size-range #f (* 10 1024 1024 1024)))
    ((1Gb-) (size-range #f (* 1 1024 1024 1024)))
    ((100Mb-) (size-range #f (* 100 1024 1024)))
    ((10Mb-) (size-range #f (* 10 1024 1024)))
    ((1Mb-) (size-range #f (* 1 1024 1024)))
    ((100Kb-) (size-range #f (* 100 1024)))
    ((10Kb-) (size-range #f (* 10 1024)))
    ((1Kb-) (size-range #f (* 1 1024)))
    (else (size-range #f #f))))

(define (load-user-list user)
  (catch/message
    (lambda ()
      (form-update-value "user"
        (let ((user-list (apply woo-list "/squidmill/users"
                                         (date-values))))
          (form-update-enum "user" user-list)
          (if (any (lambda (obj)
                      (equal? (car obj) user))
                   user-list)
            user
            (caar user-list)))))))

(define (load-backend-values)
  (catch/message
    (lambda ()
      (form-update-value-list '("follow")
        (woo-read-first "/squidmill")))))

(define (refresh-date date-type)
  (let ((status (equal? "period" date-type)))
    (form-update-visibility "date_period" status)
    (form-update-value "date_type" date-type)
    (set! *date-type* date-type)
    (not status)))

(define (refresh-user user-pat-type)
  (let ((list-status (equal? "one" user-pat-type))
        (pat-status (equal? "pattern" user-pat-type)))
    (form-update-visibility "user" list-status)
    (form-update-visibility "user_pat_label" pat-status)
    (form-update-visibility "user_pat" pat-status)
    (form-update-value "user_pat_type" user-pat-type)
    (set! *user-pat-type* user-pat-type)
    (or (and list-status (load-user-list *user*))
        (not pat-status))))

(define (refresh-uri uri-pat-type)
  (let ((status (equal? "pattern" uri-pat-type)))
    (form-update-visibility "uri_pat_label" status)
    (form-update-visibility "uri_pat" status)
    (form-update-value "uri_pat_type" uri-pat-type)
    (set! *uri-pat-type* uri-pat-type)
    (not status)))

(define (refresh-size size-type)
  (form-update-value "size_type" size-type)
  (set! *size-type* size-type)
  #f)

(define (load-local-values)
  (if (not (refresh-date *date-type*))
    (begin
      (form-update-value "start_date" (or *sdate* ""))
      (form-update-value "end_date" (or *edate* ""))))
  (if (not (refresh-user *user-pat-type*))
    (form-update-value "user_pat" (or *user-pat* "")))
  (if (not (refresh-uri *uri-pat-type*))
    (form-update-value "uri_pat" (or *uri-pat* "")))
  (refresh-size *size-type*))

(define (load-values)
  (load-backend-values)
  (load-local-values))

(define (format-amount amount)
  (if (and amount (or (number? amount)
                      (not (string-null? amount))))
    (let ((amount (or (and (number? amount) amount)
                       (string->number amount))))
      (cond
        ((> amount (* 1024 1024 1024))
         (format (string-append "~4,2f " (_ "Gb"))
                 (exact->inexact (/ amount (* 1024 1024 1024)))))
        ((> amount (* 1024 1024))
          (format (string-append "~4,2f " (_ "Mb"))
                  (exact->inexact (/ amount (* 1024 1024)))))
        ((> amount 1024)
          (format (string-append "~4,2f " (_ "Kb"))
                  (exact->inexact (/ amount 1024))))
        (else (format (string-append "~4,2f " (_ "b"))
                      (exact->inexact amount)))))
    (string-append "0.00 " (_ "b"))))

(define (update-summary)
  (catch/message
    (lambda ()
      (form-update-value "sum_size"
        (format-amount
          (woo-get-option (apply woo-read-first "/squidmill/summary"
                                 (append (date-values)
                                         (user-values)
                                         (uri-values)))
                          'sum_size))))))

(define (process-report-record item)
  (plist-fold
    (lambda (n v rlist)
      (append rlist
             (list n (if (eq? n 'size)
                       (format-amount v)
                       v))))
    '()
    item))

(define (update-report)
  (catch/message
    (lambda ()
      (form-update-enum "report"
        (map (lambda (item)
               (if (plist? item)
                 (process-report-record item)
                 (cons (car item) (process-report-record (cdr item)))))
             (apply woo-list "/squidmill"
                             (append (date-values)
                                     (user-values)
                                     (uri-values)
                                     (size-values)))))
      (let* ((reply (woo-read-first "/squidmill/report-status"))
             (status (woo-get-option reply 'report_status))
             (limit (woo-get-option reply 'limit)))
        (form-update-visibility "limit_prefix" (equal? status "100"))
        (form-update-visibility "limit" (equal? status "100"))
        (form-update-visibility "limit_postfix" (equal? status "100"))
        (form-update-value "limit" limit)))))

(define (send-follow)
  (catch/message
    (lambda ()
      (woo-write "/squidmill"
                 'follow
                 (not (eq? #f (form-value "follow")))))))

(define (query-report)
  (set! *sdate* (form-value "start_date"))
  (set! *edate* (form-value "end_date"))
  (set! *user* (form-value "user"))
  (set! *user-pat* (form-value "user_pat"))
  (set! *uri-pat* (form-value "uri_pat"))
  (update-summary)
  (update-report))

(define (init)
  (form-bind "apply" "click" send-follow)
  (form-bind "date_type" "change"
    (lambda () (refresh-date (form-value "date_type"))))
  (form-bind "user_pat_type" "change"
    (lambda () (refresh-user (form-value "user_pat_type"))))
  (form-bind "uri_pat_type" "change"
    (lambda () (refresh-uri (form-value "uri_pat_type"))))
  (form-bind "update" "click" query-report)
  (form-bind "size_type" "change"
    (lambda () (refresh-size (form-value "size_type"))
               (update-report)))
  (load-values)
  (query-report))
